/************************************************************************
*									*
* The XOLDT System							*
* Author: David S. Warren						*
* V1.0 Copyright SUNY Stony Brook, 1992					*
* This code is distributed WITHOUT ANY WARRANTY.			*
*									*
************************************************************************/

/* This is a simple example of how ``annotated'' programs might be
implemented using XOLDT and the constraint mechanism. This is taken
from a paper: ``Towards Possibilistic Logic Programming'', by Didier
Dubois, Jerome Lang and Henri Prade, which appeared in Proceedings
of the 8th International Conference on Logic Programming, Paris,
1991, pp 581-595.

The idea is simply to let xand be min and xcomp be <, so `and' takes
minimum and `or' takes maximum. */

:- use_module(subsumes).
:- ensure_loaded(xoldt).

:- op(550,fx,n).

% Interpret the empty constraint as 1.0, so necessity factors of 1.0
%		need not be written.
xand(X,[],X) :- !.
xand([],X,X) :- !.
xand(X,Y,Z) :- X<Y -> Z=X ; Z=Y.

xcomp(Op,[H1|P1],[H2|P2]) :- variant(H1,H2), comp(Op,P1,P2).

comp(Op,_,[]) :- !, Op=le.
comp(Op,[],_) :- !, Op=gt.
comp(Op,P1,P2) :- (P1 =< P2 -> Op=le ; Op=gt).

:- notable (n)/1.
n X :- xconstraint(X).
/*****************************************************/
% Examples

:- table p/0,q/0,r/0,s/0,t/0,u/0.

p :- q,r, n 0.8.
p :- s,n 0.4.
q :- t,r,n 0.3.
q :- n 0.1.
r :- n 0.5.
s :- u,r,n 0.6.
s :- n 0.2.
t.
u :- n 0.6.

% queries:
% | ?- p <- V.
% | ?- oldp(p).


% least unsure paths in a graph

:- table path/2.
:- notable edge/2.

path(X,Y) :- edge(X,Y).
path(X,Y) :- path(X,Z), edge(Z,Y).

edge(a,b) :- n 0.5.
edge(a,c) :- n 0.7.
edge(b,d) :- n 0.6.
edge(d,b) :- n 0.6.
edge(c,d) :- n 0.8.
edge(b,e) :- n 0.9.
edge(d,e) :- n 0.2.

% queries:
% | ?- path(a,T) <- V.
% | ?- oldp(path(_,_)).
