/************************************************************************
*									*
* The XOLDT System							*
* Author: David S. Warren						*
* V1.0 Copyright SUNY Stony Brook, 1992					*
* This code is distributed WITHOUT ANY WARRANTY.			*
*									*
************************************************************************/

/* This file is to be used with the XOLDT subsystem. It shows a
simple example of how it can be used to process left-recursive
grammars. The example grammar given here is the standard CF grammar
for expressions: */

:- ensure_loaded(xoldt).

/*************** Grammar 1: recognize CF expressions ****************/

% table the Prolog predicates (Prolog arity = nonterminal arity + 2).
:- table e/2, t/2.
:- notable f/2.

e --> e, [+], t.
e --> t.

t --> t, [*], f.
t --> f.

f --> [X], {integer(X)}.
f --> ['('], e, [')'].

% :- phrase(e,[1,*,2,+,3]).
%	yes
% :- phrase(e,[1,*,'(',2,+,3,')']).
%	yes
% :- phrase(e,[1,*,+,3]).
%	no

/*************** Grammar 2: parse CF expressions ****************/
% construct abstract syntax tree; notice left-associativity.

:- table e/3, t/3.
:- notable f/3.

e(plus(E,T)) --> e(E), [+], t(T).
e(T) --> t(T).

t(mult(T,F)) --> t(T), [*], f(F).
t(F) --> f(F).

f(X) --> [X], {integer(X)}.
f(E) --> ['('], e(E), [')'].

% :- phrase(e(P),[1,*,2,+,3]).
%	P = plus(mult(1,2),3) ;
%	no
% :- phrase(e(P),[1,*,'(',2,+,3,')']).
%	P = mult(1,plus(2,3)) ;
%	no
% :- phrase(e(P),[1,*,2,*,3]).
%	P = mult(mult(1,2),3) ;
%	no
% :- phrase(e(P),[1,*,+,3]).
%	no
